<?php

namespace common\repositories;

use common\helpers\Departments;
use common\models\Producer;
use yii\helpers\Html;

class ProducerRepository
{
    public function getOneById($id)
    {
        return Producer::searchOne(['id' => $id]);
    }

    public function getOneBySlug($slug)
    {
        return Producer::searchOne(['slug' => $slug]);
    }

    public function queryActive()
    {
        return Producer::find()
            ->where([
                'active' => true,
            ])
            ->orderBy('name ASC');
    }

    /**
     * Retourne le compte producteur de démonstration.
     *
     * @return Producer
     */
    public function getOneDemoAccount()
    {
        return Producer::find()->where('name LIKE \'Démo\'')->one();
    }

    /**
     * Retourne la liste des établissements pour l'initialisation d'une liste
     * sélective.
     *
     * @return array
     */
    public static function getPopulateDropdown()
    {
        $producers = Producer::find()
            ->where([
                'active' => true,
            ])
            ->orderBy('postcode, city ASC')
            ->all();

        $departments = Departments::get();
        $dataProducers = [];
        $optionsProducers = [];

        foreach ($producers as $p) {
            $departmentCode = substr($p->postcode, 0, 2);
            if (!key_exists('d' . $departmentCode, $dataProducers) && isset($departments[$departmentCode])) {
                $dataProducers['d' . $departmentCode] = '<strong>' . $departments[$departmentCode] . '</strong>';
                $optionsProducers['d' . $departmentCode] = ['disabled' => true];
            }

            $dataProducers[$p->id] = '<span class="glyphicon glyphicon-lock"></span> ' . Html::encode(
                    $p->name
                ) . ' - ' . Html::encode($p->postcode) . ' ' . Html::encode(
                    $p->city
                ) . ' <span class="glyphicon glyphicon-lock"></span>';

            if (strlen($p->code)) {
                $optionsProducers[$p->id] = ['class' => 'lock'];
            }
        }

        return [
            'data' => $dataProducers,
            'options' => $optionsProducers
        ];
    }
}